// Copyright 2003 Leigh L. Klotz, Jr.

package com.hipme.flipper;

import danger.app.AppResources;
import danger.app.Application;
import danger.app.Event;
import danger.system.CameraDevice;
import danger.ui.Bitmap;
import danger.ui.PopupMenu;
import danger.ui.ImageView;
import danger.ui.DialogWindow;
import danger.ui.ImageCodec;
import danger.ui.ImageWindow;
import danger.ui.StaticText;
import danger.util.format.StringFormat;
import danger.util.DEBUG;

/**
 * The Photo class
 */
class Photo extends Message implements Resources, Constants, Events {

  Photo() {
    super();
  }



  Photo(String userName) {
    super(userName);
  }

  public boolean receiveEvent(Event event) {
    boolean consumed = false;
    switch (event.type) {
    case kHandleFlipPhotoEvent:
      consumed = sendNewPhoto(event);
      break;
    case kShowFlipPhotoEvent:
      consumed = true;
      showSendNewPhoto(event);
      break;
    case kHandleReceivePhotoFlipShowEvent:
      consumed = true;
      handleReceivePhotoFlipShowEvent(event);
      break;
    case kHandleReceivePhotoFlipDiscardEvent:
      consumed = true;
      handleReceivePhotoFlipDiscardEvent(event);
      break;
    default:
      consumed = super.receiveEvent(event);
    }
    return consumed;
  }

  void showSendNewPhoto(Event event) {
    DEBUG.p("photo showSendNewPhoto");
    flipper app = (flipper)Application.getCurrentApp();
    DialogWindow dw = app.getDialog(kFlipPhotoDialog, this);
    ((FriendTextField)(dw.getDescendantWithID(kFlipPhotoUserName))).setText(userName);
    dw.show();
  }

  boolean sendNewPhoto(Event event) {
    DEBUG.p("Photo sendNewPhoto");
    flipper app = (flipper)Application.getCurrentApp();
    DialogWindow dw = ((DialogWindow)(event.argument));
    userName = ((FriendTextField)(dw.getDescendantWithID(kFlipPhotoUserName))).toString();
    DEBUG.p("sendNewPhoto userName = " + userName);
    // int sizeChoice = ((PopupMenu)dw.getDescendantWithID(kFlipPhotoDialogHowMenuPopupWidget)).getValueAsItemID();
    // DEBUG.p("sizeChoice id == " + sizeChoice);
    // boolean doSmallSize = (sizeChoice == kFlipPhotoDialogSizeSmall);
    CameraDevice camera = CameraDevice.getSystemCamera();
    if (camera == null) return false;
    Bitmap bitmap = camera.capture(null);
    if (bitmap == null) return false;
//    if (doSmallSize) {
//      int w = 144;
//      int h = 108;
//      DEBUG.p("Reducing size to " + w + "x" + h);
//      Bitmap origBitmap = bitmap;
//      bitmap = new Bitmap(w, h);
//      camera.generatePreviewImage(bitmap, origBitmap, 0);
//      DEBUG.p("reduced");
//    }
    int width = bitmap.getWidth();
    int height = bitmap.getHeight();
    DEBUG.p("width="+width+" height="+height);
    byte[] bytes1 = new byte[width*height/2];
    int n  = ImageCodec.encodeJPEG(bitmap, bytes1, 75);
    DEBUG.p("photo size = " + n + " bytes");
    byte[] bytes2 = new byte[n];
    System.arraycopy(bytes1, 0, bytes2, 0, n);
    DEBUG.p("userName = " + userName);
    app.setWho(userName);
    return flipOut(WHAT_NEW_PHOTO, bytes2);
  }


  Bitmap it = null;
  void handleReceiveNewPhoto(byte[] bytes, int offset) {    
    DEBUG.p("Photo handleReceiveNewPhoto offset="+offset);
    flipper app = (flipper)Application.getCurrentApp();
    app.setWho(userName);
    app.notifyAlert(userName, getNotice());
    DialogWindow dw = app.getDialog(kReceivePhotoFlipDialog, this);
    {
      StaticText st = ((StaticText)(dw.getDescendantWithID(kReceivePhotoFlipDialogUser)));
      st.setText(StringFormat.fromRsrc(kReceivePhotoFlipUserNameText, userName));
    }
    try {
      it = ImageCodec.decodeImage(bytes, offset, bytes.length-offset);
    } catch (Exception e) {
      DEBUG.p("Couldn't read photo " + e);
      e.printStackTrace();
      app.flipWindow.flipIn(userName, "(failed to read photo)");
    }
    {
      CameraDevice camera = CameraDevice.getSystemCamera();
      if (camera != null) {
	int w = 144;
	int h = 108;
	DEBUG.p("Reducing size to " + w + "x" + h);
	Bitmap preview = new Bitmap(w, h);
	camera.generatePreviewImage(preview, it, 0);
	((ImageView)(dw.getDescendantWithID(kReceivePhotoFlipDialogImagePreview))).setImage(preview);
      }
    }
    dw.show();
  }

  // Invoked from kReceivePhotoFlipDialog Show button
  void handleReceivePhotoFlipShowEvent(Event event) {
    flipper app = (flipper)Application.getCurrentApp();
    DialogWindow dw = ((DialogWindow)(event.argument));
    if (it == null) {
      DEBUG.p("No bitmap seen");
      return;
    }
    app.flipWindow.flipIn(userName, "(photo)");
    try {
      ImageWindow imagewindow = new ImageWindow(it);
      imagewindow.setFullScreen(true);
      imagewindow.show();
    } catch (Exception e) {
      DEBUG.p("Couldn't display photo " + e);
      e.printStackTrace();
      app.flipWindow.flipIn(userName, "(failed to display photo)");
    }
    handleReceivePhotoFlipDiscardEvent(event);
  }

  // Invoked from kReceivePhotoFlipDialog Discard button
  void handleReceivePhotoFlipDiscardEvent(Event event) {
    // Just in case this Photo gets held around somehow longer than expected
    flipper app = (flipper)Application.getCurrentApp();
    app.pendingQueue.dequeue();
    app.updatePreviewScreen();
    it = null;
  }

  String getNotice() {
    return userName + " has flipped you a photo.";
  }

}
